@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2013 Brazil
+  Copyright(C) 2009-2014 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -187,8 +187,12 @@ grn_str2timeval(const char *str, uint32_t str_len, grn_timeval *tv)
     return GRN_INVALID_ARGUMENT;
   }
   r1 = r2;
+  tm.tm_yday = -1;
+  tm.tm_isdst = -1;
 
-  if ((tv->tv_sec = mktime(&tm)) == -1) { return GRN_INVALID_ARGUMENT; }
+  /* tm_yday is set appropriately (0-365) on successful completion. */
+  tv->tv_sec = mktime(&tm);
+  if (tm.tm_yday == -1) { return GRN_INVALID_ARGUMENT; }
   if ((r1 + 1) < rend && *r1 == '.') { r1++; }
   uv = grn_atoi(r1, rend, &r2);
   while (r2 < r1 + 6) {
@@ -1651,36 +1655,42 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
   if (!GRN_TEXT_LEN(&buf)) { GRN_TEXT_SETS(ctx, &buf, INDEX_HTML); }
   v = GRN_TEXT_VALUE(&buf);
   grn_str_get_mime_type(ctx, v, GRN_BULK_CURR(&buf), &key_end, &filename_end);
-  if ((GRN_TEXT_LEN(&buf) >= 2 && v[0] == 'd' && v[1] == '/') &&
-      (expr = grn_ctx_get(ctx, v + 2, key_end - (v + 2))) &&
-      command_proc_p(expr)) {
-    while (p < e) {
-      int l;
-      GRN_BULK_REWIND(&buf);
-      p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIR_DELIMITER);
-      v = GRN_TEXT_VALUE(&buf);
-      l = GRN_TEXT_LEN(&buf);
-      if (l == OUTPUT_TYPE_LEN && !memcmp(v, OUTPUT_TYPE, OUTPUT_TYPE_LEN)) {
+  if ((GRN_TEXT_LEN(&buf) >= 2 && v[0] == 'd' && v[1] == '/')) {
+    const char *command_name = v + 2;
+    int command_name_size = key_end - command_name;
+    expr = grn_ctx_get(ctx, command_name, command_name_size);
+    if (expr && command_proc_p(expr)) {
+      while (p < e) {
+        int l;
         GRN_BULK_REWIND(&buf);
-        p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
+        p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIR_DELIMITER);
         v = GRN_TEXT_VALUE(&buf);
-        get_content_mime_type(ctx, v, GRN_BULK_CURR(&buf));
-      } else if (l == COMMAND_VERSION_LEN &&
-                 !memcmp(v, COMMAND_VERSION, COMMAND_VERSION_LEN)) {
-        GRN_BULK_REWIND(&buf);
-        p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
-        get_command_version(ctx, GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf));
-        if (ctx->rc) { goto exit; }
-      } else {
-        if (!(val = grn_expr_get_or_add_var(ctx, expr, v, l))) {
-          val = &buf;
+        l = GRN_TEXT_LEN(&buf);
+        if (l == OUTPUT_TYPE_LEN && !memcmp(v, OUTPUT_TYPE, OUTPUT_TYPE_LEN)) {
+          GRN_BULK_REWIND(&buf);
+          p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
+          v = GRN_TEXT_VALUE(&buf);
+          get_content_mime_type(ctx, v, GRN_BULK_CURR(&buf));
+        } else if (l == COMMAND_VERSION_LEN &&
+                   !memcmp(v, COMMAND_VERSION, COMMAND_VERSION_LEN)) {
+          GRN_BULK_REWIND(&buf);
+          p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
+          get_command_version(ctx, GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf));
+          if (ctx->rc) { goto exit; }
+        } else {
+          if (!(val = grn_expr_get_or_add_var(ctx, expr, v, l))) {
+            val = &buf;
+          }
+          grn_obj_reinit(ctx, val, GRN_DB_TEXT, 0);
+          p = grn_text_cgidec(ctx, val, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
         }
-        grn_obj_reinit(ctx, val, GRN_DB_TEXT, 0);
-        p = grn_text_cgidec(ctx, val, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
       }
+      ctx->impl->curr_expr = expr;
+      grn_expr_exec(ctx, expr, 0);
+    } else {
+      ERR(GRN_INVALID_ARGUMENT, "invalid command name: %.*s",
+          command_name_size, command_name);
     }
-    ctx->impl->curr_expr = expr;
-    grn_expr_exec(ctx, expr, 0);
   } else if ((expr = grn_ctx_get(ctx, GRN_EXPR_MISSING_NAME,
                                  strlen(GRN_EXPR_MISSING_NAME)))) {
     if ((val = grn_expr_get_var_by_offset(ctx, expr, 0))) {
