@@ -134,15 +134,6 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
 #define ERRP(ctx,lvl) \
   (((ctx) && ((grn_ctx *)(ctx))->errlvl <= (lvl)) || (grn_gctx.errlvl <= (lvl)))
 
-#define QLERR(...) do {\
-  ERRSET(ctx, GRN_WARN, GRN_INVALID_ARGUMENT, __VA_ARGS__);\
-  return F;\
-} while (0)
-
-#define QLASSERT(expr) do {\
-  if (!(expr)) { QLERR("syntax error"); }\
-} while (0)
-
 #ifdef ERR
 #  undef ERR
 #endif /* ERR */
@@ -151,6 +142,25 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
 #define MERR(...) ERRSET(ctx, GRN_ALERT, GRN_NO_MEMORY_AVAILABLE,  __VA_ARGS__)
 #define ALERT(...) ERRSET(ctx, GRN_ALERT, GRN_SUCCESS,  __VA_ARGS__)
 
+#define ERR_CAST(column, range, element) do {\
+  grn_obj inspected;\
+  char column_name[GRN_TABLE_MAX_KEY_SIZE];\
+  int column_name_size;\
+  char range_name[GRN_TABLE_MAX_KEY_SIZE];\
+  int range_name_size;\
+  GRN_TEXT_INIT(&inspected, 0);\
+  grn_inspect(ctx, &inspected, element);\
+  column_name_size = grn_obj_name(ctx, column, column_name,\
+                                  GRN_TABLE_MAX_KEY_SIZE);\
+  range_name_size = grn_obj_name(ctx, range, range_name,\
+                                 GRN_TABLE_MAX_KEY_SIZE);\
+  ERR(GRN_INVALID_ARGUMENT, "<%.*s>: failed to cast to <%.*s>: <%.*s>",\
+      column_name_size, column_name,\
+      range_name_size, range_name,\
+      (int)GRN_TEXT_LEN(&inspected), GRN_TEXT_VALUE(&inspected));\
+  GRN_OBJ_FIN(ctx, &inspected);\
+} while (0)
+
 #ifdef WIN32
 #define SERR(str) do {\
   grn_rc rc;\
@@ -383,12 +393,13 @@ GRN_VAR grn_ctx grn_gctx;
 extern int grn_pagesize;
 extern grn_critical_section grn_glock;
 extern uint32_t grn_gtick;
+extern int grn_lock_timeout;
 
 #define GRN_CTX_ALLOCATED                            (0x80)
 #define GRN_CTX_TEMPORARY_DISABLE_II_RESOLVE_SEL_AND (0x40)
 
 typedef struct {
-  int32_t tv_sec;
+  int64_t tv_sec;
   int32_t tv_nsec;
 } grn_timeval;
 
