@@ -24,8 +24,11 @@
 #include "mrb/mrb_obj.h"
 #include "mrb/mrb_column.h"
 #include "mrb/mrb_fixed_size_column.h"
+#include "mrb/mrb_variable_size_column.h"
 #include "mrb/mrb_expr.h"
 #include "mrb/mrb_accessor.h"
+#include "mrb/mrb_index_info.h"
+#include "mrb/mrb_procedure.h"
 
 #ifdef GRN_WITH_MRUBY
 static void
@@ -40,8 +43,11 @@ grn_ctx_impl_mrb_init_bindings(grn_ctx *ctx)
   grn_mrb_obj_init(ctx);
   grn_mrb_column_init(ctx);
   grn_mrb_fixed_size_column_init(ctx);
+  grn_mrb_variable_size_column_init(ctx);
   grn_mrb_expr_init(ctx);
   grn_mrb_accessor_init(ctx);
+  grn_mrb_index_info_init(ctx);
+  grn_mrb_procedure_init(ctx);
 }
 
 static void
