@@ -28,6 +28,7 @@
 #include "util.h"
 
 static grn_hash *grn_plugins = NULL;
+static grn_critical_section grn_plugins_lock;
 
 #define PATHLEN(filename) (strlen(filename) + 1)
 
@@ -51,9 +52,20 @@ static grn_hash *grn_plugins = NULL;
 #endif
 
 grn_id
-grn_plugin_get(grn_ctx *ctx, const char *filename)
+grn_plugin_reference(grn_ctx *ctx, const char *filename)
 {
-  return grn_hash_get(ctx, grn_plugins, filename, PATHLEN(filename), NULL);
+  grn_id id;
+  grn_plugin **plugin = NULL;
+
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  id = grn_hash_get(&grn_gctx, grn_plugins, filename, PATHLEN(filename),
+                    (void **)&plugin);
+  if (plugin) {
+    (*plugin)->refcount++;
+  }
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
+  return id;
 }
 
 const char *
@@ -64,7 +76,10 @@ grn_plugin_path(grn_ctx *ctx, grn_id id)
   const char *system_plugins_dir;
   size_t system_plugins_dir_size;
 
-  path = _grn_hash_key(ctx, grn_plugins, id, &key_size);
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  path = _grn_hash_key(&grn_gctx, grn_plugins, id, &key_size);
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
   if (!path) {
     return NULL;
   }
@@ -89,7 +104,7 @@ static grn_rc
 grn_plugin_call_init (grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(ctx, grn_plugins, id, &plugin)) {
+  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
     return GRN_INVALID_ARGUMENT;
   }
   if (plugin->init_func) {
@@ -102,7 +117,7 @@ static grn_rc
 grn_plugin_call_register(grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(ctx, grn_plugins, id, &plugin)) {
+  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
     return GRN_INVALID_ARGUMENT;
   }
   if (plugin->register_func) {
@@ -115,7 +130,7 @@ static grn_rc
 grn_plugin_call_fin(grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(ctx, grn_plugins, id, &plugin)) {
+  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
     return GRN_INVALID_ARGUMENT;
   }
   if (plugin->fin_func) {
@@ -165,13 +180,6 @@ grn_plugin_initialize(grn_ctx *ctx, grn_plugin *plugin,
   return ctx->rc;
 }
 
-static grn_id
-grn_plugin_find(grn_ctx *ctx, const char *filename, grn_plugin **plugin)
-{
-  return grn_hash_get(ctx, grn_plugins, filename, PATHLEN(filename),
-                      (void **)&plugin);
-}
-
 grn_id
 grn_plugin_open(grn_ctx *ctx, const char *filename)
 {
@@ -179,14 +187,15 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
   grn_dl dl;
   grn_plugin **plugin = NULL;
 
-  if ((id = grn_plugin_find(ctx, filename, plugin))) {
-    if (plugin && *plugin) {
-      (*plugin)->refcount++;
-    }
-    return id;
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  if ((id = grn_hash_get(&grn_gctx, grn_plugins, filename, PATHLEN(filename),
+                         (void **)&plugin))) {
+    (*plugin)->refcount++;
+    goto exit;
   }
+
   if ((dl = grn_dl_open(filename))) {
-    if ((id = grn_hash_add(ctx, grn_plugins, filename, PATHLEN(filename),
+    if ((id = grn_hash_add(&grn_gctx, grn_plugins, filename, PATHLEN(filename),
                            (void **)&plugin, NULL))) {
       *plugin = GRN_GMALLOCN(grn_plugin, 1);
       if (*plugin) {
@@ -196,7 +205,7 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
         }
       }
       if (!*plugin) {
-        grn_hash_delete_by_id(ctx, grn_plugins, id, NULL);
+        grn_hash_delete_by_id(&grn_gctx, grn_plugins, id, NULL);
         if (grn_dl_close(dl)) {
           /* Now, __FILE__ set in plugin is invalid. */
           ctx->errline = 0;
@@ -222,22 +231,32 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
     label = grn_dl_open_error_label();
     SERR(label);
   }
+
+exit:
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
   return id;
 }
 
 grn_rc
 grn_plugin_close(grn_ctx *ctx, grn_id id)
 {
+  grn_rc rc;
   grn_plugin *plugin;
 
   if (id == GRN_ID_NIL) {
     return GRN_INVALID_ARGUMENT;
   }
 
-  if (!grn_hash_get_value(ctx, grn_plugins, id, &plugin)) {
-    return GRN_INVALID_ARGUMENT;
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+    rc = GRN_INVALID_ARGUMENT;
+    goto exit;
+  }
+  if (--plugin->refcount) {
+    rc = GRN_SUCCESS;
+    goto exit;
   }
-  if (--plugin->refcount) { return GRN_SUCCESS; }
   grn_plugin_call_fin(ctx, id);
   if (!grn_dl_close(plugin->dl)) {
     const char *label;
@@ -245,7 +264,12 @@ grn_plugin_close(grn_ctx *ctx, grn_id id)
     SERR(label);
   }
   GRN_GFREE(plugin);
-  return grn_hash_delete_by_id(ctx, grn_plugins, id, NULL);
+  rc = grn_hash_delete_by_id(&grn_gctx, grn_plugins, id, NULL);
+
+exit:
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
+  return rc;
 }
 
 void *
@@ -258,8 +282,10 @@ grn_plugin_sym(grn_ctx *ctx, grn_id id, const char *symbol)
     return NULL;
   }
 
-  if (!grn_hash_get_value(ctx, grn_plugins, id, &plugin)) {
-    return NULL;
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+    func = NULL;
+    goto exit;
   }
   grn_dl_clear_error();
   if (!(func = grn_dl_sym(plugin->dl, symbol))) {
@@ -267,12 +293,17 @@ grn_plugin_sym(grn_ctx *ctx, grn_id id, const char *symbol)
     label = grn_dl_sym_error_label();
     SERR(label);
   }
+
+exit:
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
   return func;
 }
 
 grn_rc
 grn_plugins_init(void)
 {
+  CRITICAL_SECTION_INIT(grn_plugins_lock);
   grn_plugins = grn_hash_create(&grn_gctx, NULL, PATH_MAX, sizeof(grn_plugin *),
                                 GRN_OBJ_KEY_VAR_SIZE);
   if (!grn_plugins) { return GRN_NO_MEMORY_AVAILABLE; }
@@ -282,12 +313,14 @@ grn_plugins_init(void)
 grn_rc
 grn_plugins_fin(void)
 {
-  grn_ctx *ctx = &grn_gctx;
+  grn_rc rc;
   if (!grn_plugins) { return GRN_INVALID_ARGUMENT; }
-  GRN_HASH_EACH(ctx, grn_plugins, id, NULL, NULL, NULL, {
-      grn_plugin_close(ctx, id);
-    });
-  return grn_hash_close(&grn_gctx, grn_plugins);
+  GRN_HASH_EACH(&grn_gctx, grn_plugins, id, NULL, NULL, NULL, {
+    grn_plugin_close(&grn_gctx, id);
+  });
+  rc = grn_hash_close(&grn_gctx, grn_plugins);
+  CRITICAL_SECTION_FIN(grn_plugins_lock);
+  return rc;
 }
 
 const char *
@@ -307,19 +340,12 @@ grn_plugin_register_by_path(grn_ctx *ctx, const char *path)
   GRN_API_ENTER;
   if (GRN_DB_P(db)) {
     grn_id id;
-    grn_bool opened = GRN_FALSE;
-    id = grn_plugin_find(ctx, path, NULL);
-    if (id == GRN_ID_NIL) {
-      id = grn_plugin_open(ctx, path);
-      opened = GRN_TRUE;
-    }
+    id = grn_plugin_open(ctx, path);
     if (id) {
       ctx->impl->plugin_path = path;
       ctx->rc = grn_plugin_call_register(ctx, id);
       ctx->impl->plugin_path = NULL;
-      if (ctx->rc && opened) {
-        grn_plugin_close(ctx, id);
-      }
+      grn_plugin_close(ctx, id);
     }
   } else {
     ERR(GRN_INVALID_ARGUMENT, "invalid db assigned");
@@ -609,6 +635,27 @@ grn_plugin_proc_alloc(grn_ctx *ctx, grn_user_data *user_data,
   return grn_proc_alloc(ctx, user_data, domain, flags);
 }
 
+grn_obj *
+grn_plugin_proc_get_var(grn_ctx *ctx, grn_user_data *user_data,
+                        const char *name, int name_size)
+{
+  if (name_size < 0) {
+    if (name) {
+      name_size = strlen(name);
+    } else {
+      name_size = 0;
+    }
+  }
+  return grn_proc_get_var(ctx, user_data, name, name_size);
+}
+
+grn_obj *
+grn_plugin_proc_get_var_by_offset(grn_ctx *ctx, grn_user_data *user_data,
+                                  unsigned int offset)
+{
+  return grn_proc_get_var_by_offset(ctx, user_data, offset);
+}
+
 const char *
 grn_plugin_win32_base_dir(void)
 {
