@@ -356,7 +356,11 @@ c_output(grn_ctx *ctx)
       GRN_TEXT_INIT(&head, 0);
       GRN_TEXT_INIT(&body, GRN_OBJ_DO_SHALLOW_COPY);
       GRN_TEXT_INIT(&foot, 0);
-      GRN_TEXT_SET(ctx, &body, str, str_len);
+      if (ctx->rc == GRN_SUCCESS) {
+        GRN_TEXT_SET(ctx, &body, str, str_len);
+      } else {
+        ERR(ctx->rc, "%.*s", str_len, str);
+      }
       output_envelope(ctx, ctx->rc, &head, &body, &foot);
       fwrite(GRN_TEXT_VALUE(&head), 1, GRN_TEXT_LEN(&head), output);
       fwrite(GRN_TEXT_VALUE(&body), 1, GRN_TEXT_LEN(&body), output);
@@ -640,7 +644,6 @@ start_service(grn_ctx *ctx, const char *db_path,
 }
 
 typedef struct {
-  grn_obj body;
   grn_msg *msg;
 } ht_context;
 
@@ -650,39 +653,40 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   grn_rc expr_rc = ctx->rc;
   ht_context *hc = (ht_context *)arg;
   grn_sock fd = hc->msg->u.fd;
-  grn_obj *body = &hc->body;
-  grn_obj head, foot, *outbuf = ctx->impl->outbuf;
+  grn_obj header, head, foot, *outbuf = ctx->impl->outbuf;
   if (!(flags & GRN_CTX_TAIL)) { return; }
+  GRN_TEXT_INIT(&header, 0);
   GRN_TEXT_INIT(&head, 0);
   GRN_TEXT_INIT(&foot, 0);
-  if (!expr_rc) {
-    output_envelope(ctx, expr_rc, &head, outbuf, &foot);
-    GRN_TEXT_SETS(ctx, body, "HTTP/1.1 200 OK\r\n");
-    GRN_TEXT_PUTS(ctx, body, "Connection: close\r\n");
-    GRN_TEXT_PUTS(ctx, body, "Content-Type: ");
-    GRN_TEXT_PUTS(ctx, body, grn_ctx_get_mime_type(ctx));
-    GRN_TEXT_PUTS(ctx, body, "\r\nContent-Length: ");
-    grn_text_lltoa(ctx, body,
-                   GRN_TEXT_LEN(&head) + GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot));
-    GRN_TEXT_PUTS(ctx, body, "\r\n\r\n");
-  } else {
-    GRN_BULK_REWIND(outbuf);
-    output_envelope(ctx, expr_rc, &head, outbuf, &foot);
-    if (expr_rc == GRN_NO_SUCH_FILE_OR_DIRECTORY) {
-      GRN_TEXT_SETS(ctx, body, "HTTP/1.1 404 Not Found\r\n");
-    } else {
-      GRN_TEXT_SETS(ctx, body, "HTTP/1.1 500 Internal Server Error\r\n");
-    }
-    GRN_TEXT_PUTS(ctx, body, "Content-Type: ");
-    GRN_TEXT_PUTS(ctx, body, grn_ctx_get_mime_type(ctx));
-    GRN_TEXT_PUTS(ctx, body, "\r\n\r\n");
+  output_envelope(ctx, expr_rc, &head, outbuf, &foot);
+  switch (expr_rc) {
+  case GRN_SUCCESS :
+    GRN_TEXT_SETS(ctx, &header, "HTTP/1.1 200 OK\r\n");
+    break;
+  case GRN_INVALID_ARGUMENT :
+  case GRN_SYNTAX_ERROR :
+    GRN_TEXT_SETS(ctx, &header, "HTTP/1.1 400 Bad Request\r\n");
+    break;
+  case GRN_NO_SUCH_FILE_OR_DIRECTORY :
+    GRN_TEXT_SETS(ctx, &header, "HTTP/1.1 404 Not Found\r\n");
+    break;
+  default :
+    GRN_TEXT_SETS(ctx, &header, "HTTP/1.1 500 Internal Server Error\r\n");
+    break;
   }
+  GRN_TEXT_PUTS(ctx, &header, "Connection: close\r\n");
+  GRN_TEXT_PUTS(ctx, &header, "Content-Type: ");
+  GRN_TEXT_PUTS(ctx, &header, grn_ctx_get_mime_type(ctx));
+  GRN_TEXT_PUTS(ctx, &header, "\r\nContent-Length: ");
+  grn_text_lltoa(ctx, &header,
+                 GRN_TEXT_LEN(&head) + GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot));
+  GRN_TEXT_PUTS(ctx, &header, "\r\n\r\n");
   {
     ssize_t ret, len;
 #ifdef WIN32
     WSABUF wsabufs[4];
-    wsabufs[0].buf = GRN_TEXT_VALUE(body);
-    wsabufs[0].len = GRN_TEXT_LEN(body);
+    wsabufs[0].buf = GRN_TEXT_VALUE(&header);
+    wsabufs[0].len = GRN_TEXT_LEN(&header);
     wsabufs[1].buf = GRN_TEXT_VALUE(&head);
     wsabufs[1].len = GRN_TEXT_LEN(&head);
     wsabufs[2].buf = GRN_TEXT_VALUE(outbuf);
@@ -702,8 +706,8 @@ h_output(grn_ctx *ctx, int flags, void *arg)
     msg.msg_control = NULL;
     msg.msg_controllen = 0;
     msg.msg_flags = 0;
-    msg_iov[0].iov_base = GRN_TEXT_VALUE(body);
-    msg_iov[0].iov_len = GRN_TEXT_LEN(body);
+    msg_iov[0].iov_base = GRN_TEXT_VALUE(&header);
+    msg_iov[0].iov_len = GRN_TEXT_LEN(&header);
     msg_iov[1].iov_base = GRN_TEXT_VALUE(&head);
     msg_iov[1].iov_len = GRN_TEXT_LEN(&head);
     msg_iov[2].iov_base = GRN_TEXT_VALUE(outbuf);
@@ -714,7 +718,7 @@ h_output(grn_ctx *ctx, int flags, void *arg)
       SERR("sendmsg");
     }
 #endif /* WIN32 */
-    len = GRN_TEXT_LEN(body) + GRN_TEXT_LEN(&head) +
+    len = GRN_TEXT_LEN(&header) + GRN_TEXT_LEN(&head) +
       GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot);
     if (ret != len) {
       GRN_LOG(&grn_gctx, GRN_LOG_NOTICE,
@@ -722,10 +726,10 @@ h_output(grn_ctx *ctx, int flags, void *arg)
               (long long int)ret, (long long int)len);
     }
   }
-  GRN_BULK_REWIND(body);
   GRN_BULK_REWIND(outbuf);
   GRN_OBJ_FIN(ctx, &foot);
   GRN_OBJ_FIN(ctx, &head);
+  GRN_OBJ_FIN(ctx, &header);
 }
 
 static void
@@ -1337,13 +1341,34 @@ enum {
   EDGE_ABORT = 0x03,
 };
 
+static void
+check_rlimit_nofile(grn_ctx *ctx)
+{
+#ifndef WIN32
+  struct rlimit limit;
+  limit.rlim_cur = 0;
+  limit.rlim_max = 0;
+  getrlimit(RLIMIT_NOFILE, &limit);
+  if (limit.rlim_cur < RLIMIT_NOFILE_MINIMUM) {
+    limit.rlim_cur = RLIMIT_NOFILE_MINIMUM;
+    limit.rlim_max = RLIMIT_NOFILE_MINIMUM;
+    setrlimit(RLIMIT_NOFILE, &limit);
+    limit.rlim_cur = 0;
+    limit.rlim_max = 0;
+    getrlimit(RLIMIT_NOFILE, &limit);
+  }
+  GRN_LOG(ctx, GRN_LOG_NOTICE,
+          "RLIMIT_NOFILE(%" GRN_FMT_LLD ",%" GRN_FMT_LLD ")",
+          (long long int)limit.rlim_cur, (long long int)limit.rlim_max);
+#endif /* WIN32 */
+}
+
 static void * CALLBACK
 h_worker(void *arg)
 {
   ht_context hc;
   grn_ctx ctx_, *ctx = &ctx_;
   grn_ctx_init(ctx, 0);
-  GRN_TEXT_INIT(&hc.body, 0);
   grn_ctx_use(ctx, (grn_obj *)arg);
   grn_ctx_recv_handler_set(ctx, h_output, &hc);
   GRN_LOG(&grn_gctx, GRN_LOG_NOTICE, "thread start (%d/%d)", nfthreads, nthreads + 1);
@@ -1368,7 +1393,6 @@ exit :
   nthreads--;
   MUTEX_UNLOCK(q_mutex);
   GRN_LOG(&grn_gctx, GRN_LOG_NOTICE, "thread end (%d/%d)", nfthreads, nthreads);
-  GRN_OBJ_FIN(ctx, &hc.body);
   grn_ctx_fin(ctx);
   return NULL;
 }
@@ -1409,20 +1433,7 @@ h_server(char *path)
   CRITICAL_SECTION_INIT(cache_lock);
   GRN_COM_QUEUE_INIT(&ctx_new);
   GRN_COM_QUEUE_INIT(&ctx_old);
-#ifndef WIN32
-  {
-    struct rlimit lim;
-    lim.rlim_cur = 4096;
-    lim.rlim_max = 4096;
-    /* RLIMIT_OFILE */
-    setrlimit(RLIMIT_NOFILE, &lim);
-    lim.rlim_cur = 0;
-    lim.rlim_max = 0;
-    getrlimit(RLIMIT_NOFILE, &lim);
-    GRN_LOG(ctx, GRN_LOG_NOTICE, "RLIMIT_NOFILE(%" GRN_FMT_LLD ",%" GRN_FMT_LLD ")",
-            (long long int)lim.rlim_cur, (long long int)lim.rlim_max);
-  }
-#endif /* WIN32 */
+  check_rlimit_nofile(ctx);
   exit_code = start_service(ctx, path, NULL, h_handler);
   grn_ctx_fin(ctx);
   return exit_code;
@@ -1519,7 +1530,7 @@ g_output(grn_ctx *ctx, int flags, void *arg)
   msg->header.proto = req->header.proto == GRN_COM_PROTO_MBREQ
     ? GRN_COM_PROTO_MBRES : req->header.proto;
   if (ctx->rc != GRN_SUCCESS && GRN_BULK_VSIZE(ctx->impl->outbuf) == 0) {
-    grn_ctx_output_cstr(ctx, ctx->errbuf);
+    GRN_TEXT_PUTS(ctx, ctx->impl->outbuf, ctx->errbuf);
   }
   if (grn_msg_send(ctx, (grn_obj *)msg,
                    (flags & GRN_CTX_MORE) ? GRN_CTX_MORE : GRN_CTX_TAIL)) {
@@ -1583,25 +1594,7 @@ g_server(char *path)
   CRITICAL_SECTION_INIT(cache_lock);
   GRN_COM_QUEUE_INIT(&ctx_new);
   GRN_COM_QUEUE_INIT(&ctx_old);
-#ifndef WIN32
-  {
-    struct rlimit limit;
-    limit.rlim_cur = 0;
-    limit.rlim_max = 0;
-    getrlimit(RLIMIT_NOFILE, &limit);
-    if (limit.rlim_cur < RLIMIT_NOFILE_MINIMUM) {
-      limit.rlim_cur = RLIMIT_NOFILE_MINIMUM;
-      limit.rlim_max = RLIMIT_NOFILE_MINIMUM;
-      setrlimit(RLIMIT_NOFILE, &limit);
-      limit.rlim_cur = 0;
-      limit.rlim_max = 0;
-      getrlimit(RLIMIT_NOFILE, &limit);
-    }
-    GRN_LOG(ctx, GRN_LOG_NOTICE,
-            "RLIMIT_NOFILE(%" GRN_FMT_LLD ",%" GRN_FMT_LLD ")",
-            (long long int)limit.rlim_cur, (long long int)limit.rlim_max);
-  }
-#endif /* WIN32 */
+  check_rlimit_nofile(ctx);
   exit_code = start_service(ctx, path, g_dispatcher, g_handler);
   grn_ctx_fin(ctx);
   return exit_code;
